"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.constructRDSGlobalAmplifyInput = exports.getRDSGlobalAmplifyInput = exports.validateRDSInputDBConfig = exports.getRDSDBConfigFromAmplifyInput = exports.readRDSGlobalAmplifyInput = exports.constructDefaultGlobalAmplifyInput = void 0;
const graphql_1 = require("graphql");
const fs = __importStar(require("fs-extra"));
const amplify_cli_core_1 = require("amplify-cli-core");
const lodash_1 = __importDefault(require("lodash"));
const import_appsync_api_types_1 = require("../types/import-appsync-api-types");
const getGlobalAmplifyInputEntries = async (context, dataSourceType = import_appsync_api_types_1.ImportedRDSType.MYSQL, includeAuthRule = true) => {
    const inputs = [
        {
            name: 'engine',
            type: 'String',
            default: dataSourceType,
        },
        {
            name: 'host',
            type: 'String',
            default: 'ENTER YOUR DATABASE HOSTNAME HERE',
        },
        {
            name: 'port',
            type: 'Int',
            default: 3306,
            comment: 'ENTER PORT NUMBER HERE',
        },
        {
            name: 'database',
            type: 'String',
            default: 'ENTER YOUR DATABASE NAME HERE',
        },
    ];
    if (includeAuthRule && (await amplify_cli_core_1.ApiCategoryFacade.getTransformerVersion(context) === 2)) {
        const authDocLink = (0, amplify_cli_core_1.getGraphQLTransformerAuthDocLink)(2);
        inputs.push({
            name: 'globalAuthRule',
            type: 'AuthRule',
            default: '{ allow: public }',
            comment: `This "input" configures a global authorization rule to enable public access to all models in this schema. Learn more about authorization rules here:${authDocLink}`
        });
    }
    ;
    return inputs;
};
const constructDefaultGlobalAmplifyInput = async (context, dataSourceType) => {
    const inputs = await getGlobalAmplifyInputEntries(context, dataSourceType);
    const inputsString = inputs.reduce((acc, input) => acc + ` ${input.name}: ${input.type} = ${input.type === 'String' ? '"' + input.default + '"' : input.default} ${input.comment ? '# ' + input.comment : ''} \n`, '');
    return `input Amplify {\n${inputsString}}\n`;
};
exports.constructDefaultGlobalAmplifyInput = constructDefaultGlobalAmplifyInput;
const readRDSGlobalAmplifyInput = async (pathToSchemaFile) => {
    const schemaContent = fs.readFileSync(pathToSchemaFile, 'utf-8');
    if (lodash_1.default.isEmpty(schemaContent === null || schemaContent === void 0 ? void 0 : schemaContent.replace(/[\r\n]/gm, ''))) {
        throw new Error('The schema file is empty');
    }
    const parsedSchema = (0, graphql_1.parse)(schemaContent);
    return parsedSchema.definitions.find((definition) => definition.kind === 'InputObjectTypeDefinition' &&
        definition.name &&
        definition.name.value === 'Amplify');
};
exports.readRDSGlobalAmplifyInput = readRDSGlobalAmplifyInput;
const getRDSDBConfigFromAmplifyInput = async (context, inputNode) => {
    const expectedInputs = (await getGlobalAmplifyInputEntries(context, import_appsync_api_types_1.ImportedRDSType.MYSQL, false)).map(item => item.name);
    const inputs = {};
    expectedInputs.map((input) => {
        var _a, _b, _c;
        const value = (_c = (_b = (_a = inputNode === null || inputNode === void 0 ? void 0 : inputNode.fields) === null || _a === void 0 ? void 0 : _a.find((field) => { var _a; return ((_a = field === null || field === void 0 ? void 0 : field.name) === null || _a === void 0 ? void 0 : _a.value) === input; })) === null || _b === void 0 ? void 0 : _b.defaultValue) === null || _c === void 0 ? void 0 : _c.value;
        if (lodash_1.default.isEmpty(value)) {
            throw new Error(`Invalid value for ${input} input in the GraphQL schema. Correct and re-try.`);
        }
        inputs[input] = value;
    });
    return inputs;
};
exports.getRDSDBConfigFromAmplifyInput = getRDSDBConfigFromAmplifyInput;
const validateRDSInputDBConfig = async (context, config) => {
    const expectedInputs = (await getGlobalAmplifyInputEntries(context, import_appsync_api_types_1.ImportedRDSType.MYSQL, false)).map((item) => item.name);
    const missingInputs = expectedInputs.filter((input) => lodash_1.default.isEmpty(config[input]));
    if (!lodash_1.default.isEmpty(missingInputs)) {
        throw new Error(`The database connection parameters: ${missingInputs.join(',')}, are missing. Specify them in the schema and re-run.`);
    }
    const invalidInputs = expectedInputs.filter((input) => { var _a; return ((_a = config[input]) === null || _a === void 0 ? void 0 : _a.indexOf(' ')) >= 0; });
    if (!lodash_1.default.isEmpty(invalidInputs)) {
        throw new Error(`The database connection parameters: ${invalidInputs.join(',')}, might be invalid. Correct them in the schema and re-run.`);
    }
};
exports.validateRDSInputDBConfig = validateRDSInputDBConfig;
const getRDSGlobalAmplifyInput = async (context, pathToSchemaFile) => {
    const inputNode = await (0, exports.readRDSGlobalAmplifyInput)(pathToSchemaFile);
    const config = await (0, exports.getRDSDBConfigFromAmplifyInput)(context, inputNode);
    await (0, exports.validateRDSInputDBConfig)(context, config);
    return inputNode;
};
exports.getRDSGlobalAmplifyInput = getRDSGlobalAmplifyInput;
const constructRDSGlobalAmplifyInput = async (context, config, inputNode) => {
    const rdsConfig = config;
    const expectedInputs = (await getGlobalAmplifyInputEntries(context, import_appsync_api_types_1.ImportedRDSType.MYSQL, false)).map(item => item.name);
    expectedInputs.forEach((input) => {
        var _a;
        const inputNodeField = (_a = inputNode === null || inputNode === void 0 ? void 0 : inputNode.fields) === null || _a === void 0 ? void 0 : _a.find((field) => { var _a; return ((_a = field === null || field === void 0 ? void 0 : field.name) === null || _a === void 0 ? void 0 : _a.value) === input; });
        if (inputNodeField && rdsConfig[input]) {
            inputNodeField.defaultValue.value = rdsConfig[input];
        }
    });
    const result = (0, graphql_1.print)(inputNode);
    return result;
};
exports.constructRDSGlobalAmplifyInput = constructRDSGlobalAmplifyInput;
//# sourceMappingURL=rds-input-utils.js.map